import cv2

from mower.solvers.fight.mixin import FightMixin
from mower.utils import config
from mower.utils.graph import SceneGraphSolver
from mower.utils.image import cropimg, thres2
from mower.utils.scene import Scene
from mower.utils.vector import va


class StrongholdProtocol1(SceneGraphSolver, FightMixin):
    solver_name = "卫戍协议刷分"

    def deploy(self):
        pos = (1058, 746) if self.find("stronghold_protocol/block") else (1475, 524)
        self.swipe_ext([(1602, 989), pos], [config.conf.first_swipe_duration], 100, 0.5)
        for _ in range(5):
            if not (flag := self.drag_success()):
                self.sleep(0.1)
            else:
                break
        if flag:
            self.swipe_ext(
                [pos, va(pos, (300, 0))], [config.conf.second_swipe_duration], 0, 0.4
            )
            if self.drag_success():
                self.ctap(va(pos, (-300, -300)), 0.5, config.screenshot_avg / 1000)
        else:
            self.ctap(va(pos, (-300, -300)), 0.5, config.screenshot_avg / 1000)

    def transition(self):
        if (scene := self.scene()) == Scene.TERMINAL_MAIN:
            self.terminal_entry("stronghold_protocol/entry.jpg")
        elif scene == Scene.SP_MAIN:
            if self.find("stronghold_protocol/level50"):
                return True
            if pos := self.find("stronghold_protocol/stop"):
                if not self.find("stronghold_protocol/round1"):
                    self.tap(pos)
                    return
            self.tap((1664, 911))
        elif scene == Scene.SP_DEFENCE:
            if pos := self.find("stronghold_protocol/start"):
                self.tap(pos)
                return
            self.tap((383, 948))
        elif scene == Scene.SP_ACTION:
            # 第二轮退出
            img = cropimg(config.recog.gray, ((1045, 24), (1063, 46)))
            img = cv2.bitwise_not(thres2(img, 127))
            if config.recog.num.number_int("secret_front", img=img) >= 2:
                self.back()
                return

            # 购买干员
            if pos := self.find("stronghold_protocol/buy1"):
                if self.find("stronghold_protocol/medic"):
                    self.tap((804, 513))
                    return
                self.tap(pos)
                return
            if pos := self.find("stronghold_protocol/buy2"):
                if self.find("stronghold_protocol/medic"):
                    self.tap((875, 431))
                    return
                self.tap(pos)
                return
            if pos := self.find("stronghold_protocol/buy3"):
                self.tap(pos)
                return

            if self.find("stronghold_protocol/empty"):
                if self.find("stronghold_protocol/money0") or self.find(
                    "stronghold_protocol/money2"
                ):
                    self.tap((1827, 93))  # 开始作战
                    return
                self.tap((407, 540))  # 购买干员
                return

            # 放置干员
            self.deploy()

        else:
            self.scene_graph_step(Scene.TERMINAL_MAIN)


class StrongholdProtocol2(SceneGraphSolver, FightMixin):
    solver_name = "卫戍协议刷合成"

    def deploy(self):
        pos = (1475, 524)
        self.swipe_ext([(1602, 989), pos], [config.conf.first_swipe_duration], 100, 0.5)
        for _ in range(5):
            if not (flag := self.drag_success()):
                self.sleep(0.1)
            else:
                break
        if flag:
            self.swipe_ext(
                [pos, va(pos, (300, 0))], [config.conf.second_swipe_duration], 0, 0.4
            )
            if self.drag_success():
                self.ctap(va(pos, (-300, -300)), 0.5, config.screenshot_avg / 1000)
        else:
            self.ctap(va(pos, (-300, -300)), 0.5, config.screenshot_avg / 1000)

    def transition(self):
        if (scene := self.scene()) == Scene.TERMINAL_MAIN:
            self.terminal_entry("stronghold_protocol/entry.jpg")
        elif scene == Scene.SP_MAIN:
            if pos := self.find("stronghold_protocol/stop"):
                if not self.find("stronghold_protocol/round1"):
                    self.tap(pos)
                    return
            self.tap((1664, 911))
        elif scene == Scene.SP_DEFENCE:
            if pos := self.find("stronghold_protocol/start"):
                self.tap(pos)
                return
            self.tap((138, 950))
        elif scene == Scene.SP_ACTION:
            img = cropimg(config.recog.gray, ((1045, 24), (1063, 46)))
            img = cv2.bitwise_not(thres2(img, 127))
            round = config.recog.num.number_int("secret_front", img=img)

            if round > 2:  # 第三轮以后直接退出
                self.back()
                return
            if pos := self.find("stronghold_protocol/buy2"):
                self.tap(pos)  # 确认购买
                return
            if round == 1:
                if self.find("stronghold_protocol/empty"):
                    if self.find("stronghold_protocol/money2"):
                        self.tap((1827, 93))  # 开始作战
                        return
                    self.tap((302, 540))  # 购买泡普卡
                    return
                self.deploy()
                return
            if pos := self.find("stronghold_protocol/refresh"):
                self.tap(pos)
                return
            if self.find("stronghold_protocol/money10"):
                self.tap((627, 545))  # 刷新
                return
            if pos := self.find("stronghold_protocol/buy2"):
                self.tap(pos)
                return
            if self.find("stronghold_protocol/money6"):
                self.back()
                return
            self.tap((302, 540))
        else:
            self.scene_graph_step(Scene.TERMINAL_MAIN)
